@@ -1,4 +1,4 @@
-# Copyright 2015 Google Inc. All Rights Reserved.
+# Copyright 2015-2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -38,7 +38,7 @@ from yapf.yapflib import py3compat
 from yapf.yapflib import style
 from yapf.yapflib import yapf_api
 
-__version__ = '0.8.2'
+__version__ = '0.20.1'
 
 
 def main(argv):
@@ -55,26 +55,30 @@ def main(argv):
     YapfError: if none of the supplied files were Python files.
   """
   parser = argparse.ArgumentParser(description='Formatter for Python code.')
-  parser.add_argument('-v',
-                      '--version',
-                      action='store_true',
-                      help='show version number and exit')
+  parser.add_argument(
+      '-v',
+      '--version',
+      action='store_true',
+      help='show version number and exit')
 
   diff_inplace_group = parser.add_mutually_exclusive_group()
-  diff_inplace_group.add_argument('-d',
-                                  '--diff',
-                                  action='store_true',
-                                  help='print the diff for the fixed source')
-  diff_inplace_group.add_argument('-i',
-                                  '--in-place',
-                                  action='store_true',
-                                  help='make changes to files in place')
+  diff_inplace_group.add_argument(
+      '-d',
+      '--diff',
+      action='store_true',
+      help='print the diff for the fixed source')
+  diff_inplace_group.add_argument(
+      '-i',
+      '--in-place',
+      action='store_true',
+      help='make changes to files in place')
 
   lines_recursive_group = parser.add_mutually_exclusive_group()
-  lines_recursive_group.add_argument('-r',
-                                     '--recursive',
-                                     action='store_true',
-                                     help='run recursively over directories')
+  lines_recursive_group.add_argument(
+      '-r',
+      '--recursive',
+      action='store_true',
+      help='run recursively over directories')
   lines_recursive_group.add_argument(
       '-l',
       '--lines',
@@ -83,30 +87,44 @@ def main(argv):
       default=None,
       help='range of lines to reformat, one-based')
 
-  parser.add_argument('-e',
-                      '--exclude',
-                      metavar='PATTERN',
-                      action='append',
-                      default=None,
-                      help='patterns for files to exclude from formatting')
+  parser.add_argument(
+      '-e',
+      '--exclude',
+      metavar='PATTERN',
+      action='append',
+      default=None,
+      help='patterns for files to exclude from formatting')
   parser.add_argument(
       '--style',
       action='store',
       help=('specify formatting style: either a style name (for example "pep8" '
             'or "google"), or the name of a file with style settings. The '
-            'default is pep8 unless a %s or %s file located in one of the '
-            'parent directories of the source file (or current directory for '
-            'stdin)' % (style.LOCAL_STYLE, style.SETUP_CONFIG)))
-  parser.add_argument('--style-help',
-                      action='store_true',
-                      help='show style settings and exit')
-  parser.add_argument('--no-local-style',
-                      action='store_true',
-                      help="don't search for local style definition")
-  parser.add_argument('--verify',
-                      action='store_true',
-                      help=('try to verify reformatted code for syntax errors. '
-                            'This option is for debugging purposes only.'))
+            'default is pep8 unless a %s or %s file located in the same '
+            'directory as the source or one of its parent directories '
+            '(for stdin, the current directory is used).' %
+            (style.LOCAL_STYLE, style.SETUP_CONFIG)))
+  parser.add_argument(
+      '--style-help',
+      action='store_true',
+      help=('show style settings and exit; this output can be '
+            'saved to .style.yapf to make your settings '
+            'permanent'))
+  parser.add_argument(
+      '--no-local-style',
+      action='store_true',
+      help="don't search for local style definition")
+  parser.add_argument('--verify', action='store_true', help=argparse.SUPPRESS)
+  parser.add_argument(
+      '-p',
+      '--parallel',
+      action='store_true',
+      help=('Run yapf in parallel when formatting multiple files. Requires '
+            'concurrent.futures in Python 2.X'))
+  parser.add_argument(
+      '-vv',
+      '--verbose',
+      action='store_true',
+      help='Print out file names while processing')
 
   parser.add_argument('files', nargs='*')
   args = parser.parse_args(argv[1:])
@@ -117,10 +135,11 @@ def main(argv):
 
   if args.style_help:
     style.SetGlobalStyle(style.CreateStyleFromConfig(args.style))
+    print('[style]')
     for option, docstring in sorted(style.Help().items()):
-      print(option, '=', style.Get(option), sep='')
       for line in docstring.splitlines():
-        print('  ', line)
+        print('#', line and ' ' or '', line, sep='')
+      print(option.lower(), '=', style.Get(option), sep='')
       print()
     return 0
 
@@ -136,6 +155,8 @@ def main(argv):
 
     original_source = []
     while True:
+      if sys.stdin.closed:
+        break
       try:
         # Use 'raw_input' instead of 'sys.stdin.read', because otherwise the
         # user will need to hit 'Ctrl-D' more than once if they're inputting
@@ -144,30 +165,37 @@ def main(argv):
         original_source.append(py3compat.raw_input())
       except EOFError:
         break
+
     style_config = args.style
     if style_config is None and not args.no_local_style:
       style_config = file_resources.GetDefaultStyleForDir(os.getcwd())
-    reformatted_source, changed = yapf_api.FormatCode(
-        py3compat.unicode('\n'.join(original_source) + '\n'),
+
+    source = [line.rstrip() for line in original_source]
+    reformatted_source, _ = yapf_api.FormatCode(
+        py3compat.unicode('\n'.join(source) + '\n'),
         filename='<stdin>',
         style_config=style_config,
         lines=lines,
         verify=args.verify)
-    sys.stdout.write(reformatted_source)
-    return 2 if changed else 0
+    file_resources.WriteReformattedCode('<stdout>', reformatted_source)
+    return 0
 
   files = file_resources.GetCommandLineFiles(args.files, args.recursive,
                                              args.exclude)
   if not files:
     raise errors.YapfError('Input filenames did not match any python files')
-  changed = FormatFiles(files,
-                        lines,
-                        style_config=args.style,
-                        no_local_style=args.no_local_style,
-                        in_place=args.in_place,
-                        print_diff=args.diff,
-                        verify=args.verify)
-  return 2 if changed else 0
+
+  changed = FormatFiles(
+      files,
+      lines,
+      style_config=args.style,
+      no_local_style=args.no_local_style,
+      in_place=args.in_place,
+      print_diff=args.diff,
+      verify=args.verify,
+      parallel=args.parallel,
+      verbose=args.verbose)
+  return 1 if changed and args.diff else 0
 
 
 def FormatFiles(filenames,
@@ -176,7 +204,9 @@ def FormatFiles(filenames,
                 no_local_style=False,
                 in_place=False,
                 print_diff=False,
-                verify=True):
+                verify=False,
+                parallel=False,
+                verbose=False):
   """Format a list of files.
 
   Arguments:
@@ -192,35 +222,63 @@ def FormatFiles(filenames,
     print_diff: (bool) Instead of returning the reformatted source, return a
       diff that turns the formatted source into reformatter source.
     verify: (bool) True if reformatted code should be verified for syntax.
+    parallel: (bool) True if should format multiple files in parallel.
+    verbose: (bool) True if should print out filenames while processing.
 
   Returns:
     True if the source code changed in any of the files being formatted.
   """
   changed = False
-  for filename in filenames:
-    logging.info('Reformatting %s', filename)
-    if style_config is None and not no_local_style:
-      style_config = (
-          file_resources.GetDefaultStyleForDir(os.path.dirname(filename)))
-    try:
-      reformatted_code, encoding, has_change = yapf_api.FormatFile(
-          filename,
-          in_place=in_place,
-          style_config=style_config,
-          lines=lines,
-          print_diff=print_diff,
-          verify=verify,
-          logger=logging.warning)
-      if has_change and reformatted_code is not None:
-        file_resources.WriteReformattedCode(filename, reformatted_code,
-                                            in_place, encoding)
-      changed |= has_change
-    except SyntaxError as e:
-      e.filename = filename
-      raise
+  if parallel:
+    import multiprocessing  # pylint: disable=g-import-not-at-top
+    import concurrent.futures  # pylint: disable=g-import-not-at-top
+    workers = min(multiprocessing.cpu_count(), len(filenames))
+    with concurrent.futures.ProcessPoolExecutor(workers) as executor:
+      future_formats = [
+          executor.submit(_FormatFile, filename, lines, style_config,
+                          no_local_style, in_place, print_diff, verify, verbose)
+          for filename in filenames
+      ]
+      for future in concurrent.futures.as_completed(future_formats):
+        changed |= future.result()
+  else:
+    for filename in filenames:
+      changed |= _FormatFile(filename, lines, style_config, no_local_style,
+                             in_place, print_diff, verify, verbose)
   return changed
 
 
+def _FormatFile(filename,
+                lines,
+                style_config=None,
+                no_local_style=False,
+                in_place=False,
+                print_diff=False,
+                verify=False,
+                verbose=False):
+  if verbose:
+    print('Reformatting %s' % filename)
+  if style_config is None and not no_local_style:
+    style_config = file_resources.GetDefaultStyleForDir(
+        os.path.dirname(filename))
+  try:
+    reformatted_code, encoding, has_change = yapf_api.FormatFile(
+        filename,
+        in_place=in_place,
+        style_config=style_config,
+        lines=lines,
+        print_diff=print_diff,
+        verify=verify,
+        logger=logging.warning)
+    if not in_place and reformatted_code:
+      file_resources.WriteReformattedCode(filename, reformatted_code, encoding,
+                                          in_place)
+    return has_change
+  except SyntaxError as e:
+    e.filename = filename
+    raise
+
+
 def _GetLines(line_strings):
   """Parses the start and end lines from a line string like 'start-end'.
 
