@@ -14,10 +14,11 @@
 """Utilities for Python2 / Python3 compatibility."""
 
 import io
+import os
 import sys
 
-PY3 = sys.version_info[0] == 3
-PY36 = sys.version_info[0] == 3 and sys.version_info[1] >= 6
+PY3 = sys.version_info[0] >= 3
+PY36 = sys.version_info[0] >= 3 and sys.version_info[1] >= 6
 
 if PY3:
   StringIO = io.StringIO
@@ -33,7 +34,10 @@ if PY3:
 
   range = range
   ifilter = filter
-  raw_input = input
+
+  def raw_input():
+    wrapper = io.TextIOWrapper(sys.stdin.buffer, encoding='utf-8')
+    return wrapper.buffer.raw.readall().decode('utf-8')
 
   import configparser
 
@@ -76,16 +80,29 @@ def EncodeAndWriteToStdout(s, encoding='utf-8'):
     encoding: (string) The encoding of the string.
   """
   if PY3:
-    sys.stdout.buffer.write(codecs.encode(s, encoding))
+    sys.stdout.buffer.write(s.encode(encoding))
+  elif sys.platform == 'win32':
+    # On python 2 and Windows universal newline transformation will be in
+    # effect on stdout. Python 2 will not let us avoid the easily because
+    # it happens based on whether the file handle is opened in O_BINARY or
+    # O_TEXT state. However we can tell Windows itself to change the current
+    # mode, and python 2 will follow suit. However we must take care to change
+    # the mode on the actual external stdout not just the current sys.stdout
+    # which may have been monkey-patched inside the python environment.
+    import msvcrt  # pylint: disable=g-import-not-at-top
+    if sys.__stdout__ is sys.stdout:
+      msvcrt.setmode(sys.stdout.fileno(), os.O_BINARY)
+    sys.stdout.write(s.encode(encoding))
   else:
     sys.stdout.write(s.encode(encoding))
 
 
-def unicode(s):
-  """Force conversion of s to unicode."""
-  if PY3:
-    return s
-  else:
+if PY3:
+  unicode = str  # pylint: disable=redefined-builtin,invalid-name
+else:
+
+  def unicode(s):  # pylint: disable=invalid-name
+    """Force conversion of s to unicode."""
     return __builtin__.unicode(s, 'utf-8')
 
 
