@@ -1,4 +1,4 @@
-# Copyright 2015 Google Inc. All Rights Reserved.
+# Copyright 2015-2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -36,25 +36,29 @@ from yapf.yapflib import style
 from yapf.yapflib import verifier
 
 
-def Reformat(uwlines, verify=True):
+def Reformat(uwlines, verify=False, lines=None):
   """Reformat the unwrapped lines.
 
   Arguments:
     uwlines: (list of unwrapped_line.UnwrappedLine) Lines we want to format.
     verify: (bool) True if reformatted code should be verified for syntax.
+    lines: (set of int) The lines which can be modified or None if there is no
+      line range restriction.
 
   Returns:
     A string representing the reformatted code.
   """
   final_lines = []
   prev_uwline = None  # The previous line.
+  indent_width = style.Get('INDENT_WIDTH')
 
   for uwline in _SingleOrMergedLines(uwlines):
     first_token = uwline.first
-    _FormatFirstToken(first_token, uwline.depth, prev_uwline)
+    _FormatFirstToken(first_token, uwline.depth, prev_uwline, final_lines)
 
-    indent_amt = style.Get('INDENT_WIDTH') * uwline.depth
+    indent_amt = indent_width * uwline.depth
     state = format_decision_state.FormatDecisionState(uwline, indent_amt)
+    state.MoveStateToNextToken()
 
     if not uwline.disable:
       if uwline.first.is_comment:
@@ -64,42 +68,34 @@ def Reformat(uwlines, verify=True):
       if prev_uwline and prev_uwline.disable:
         # Keep the vertical spacing between a disabled and enabled formatting
         # region.
-        _RetainVerticalSpacing(prev_uwline, uwline)
+        _RetainRequiredVerticalSpacingBetweenTokens(uwline.first,
+                                                    prev_uwline.last, lines)
+      if any(tok.is_comment for tok in uwline.tokens):
+        _RetainVerticalSpacingBeforeComments(uwline)
 
     if (_LineContainsI18n(uwline) or uwline.disable or
         _LineHasContinuationMarkers(uwline)):
       _RetainHorizontalSpacing(uwline)
-      _RetainVerticalSpacing(prev_uwline, uwline)
+      _RetainRequiredVerticalSpacing(uwline, prev_uwline, lines)
       _EmitLineUnformatted(state)
-    elif _CanPlaceOnSingleLine(uwline):
+    elif _CanPlaceOnSingleLine(uwline) and not any(tok.must_split
+                                                   for tok in uwline.tokens):
       # The unwrapped line fits on one line.
       while state.next_token:
         state.AddTokenToState(newline=False, dry_run=False)
     else:
-      if not _AnalyzeSolutionSpace(state, dry_run=False):
+      if not _AnalyzeSolutionSpace(state):
         # Failsafe mode. If there isn't a solution to the line, then just emit
         # it as is.
         state = format_decision_state.FormatDecisionState(uwline, indent_amt)
+        state.MoveStateToNextToken()
         _RetainHorizontalSpacing(uwline)
-        _RetainVerticalSpacing(prev_uwline, uwline)
+        _RetainRequiredVerticalSpacing(uwline, prev_uwline, None)
         _EmitLineUnformatted(state)
 
     final_lines.append(uwline)
     prev_uwline = uwline
-
-  formatted_code = []
-  for line in final_lines:
-    formatted_line = []
-    for tok in line.tokens:
-      if tok.name in pytree_utils.NONSEMANTIC_TOKENS:
-        continue
-      formatted_line.append(tok.whitespace_prefix)
-      formatted_line.append(tok.value)
-    formatted_code.append(''.join(formatted_line))
-    if verify:
-      verifier.VerifyCode(formatted_code[-1])
-
-  return ''.join(formatted_code) + '\n'
+  return _FormatFinalLines(final_lines, verify)
 
 
 def _RetainHorizontalSpacing(uwline):
@@ -108,20 +104,61 @@ def _RetainHorizontalSpacing(uwline):
     tok.RetainHorizontalSpacing(uwline.first.column, uwline.depth)
 
 
-def _RetainVerticalSpacing(prev_uwline, cur_uwline):
-  """Retain all vertical spacing between lines."""
-  if not prev_uwline:
+def _RetainRequiredVerticalSpacing(cur_uwline, prev_uwline, lines):
+  prev_tok = None
+  if prev_uwline is not None:
+    prev_tok = prev_uwline.last
+  for cur_tok in cur_uwline.tokens:
+    _RetainRequiredVerticalSpacingBetweenTokens(cur_tok, prev_tok, lines)
+    prev_tok = cur_tok
+
+
+def _RetainRequiredVerticalSpacingBetweenTokens(cur_tok, prev_tok, lines):
+  """Retain vertical spacing between two tokens if not in editable range."""
+  if prev_tok is None:
     return
-  if prev_uwline.last.is_string:
-    prev_lineno = prev_uwline.last.lineno + prev_uwline.last.value.count('\n')
+
+  if prev_tok.is_string:
+    prev_lineno = prev_tok.lineno + prev_tok.value.count('\n')
+  elif prev_tok.is_pseudo_paren:
+    if not prev_tok.previous_token.is_multiline_string:
+      prev_lineno = prev_tok.previous_token.lineno
+    else:
+      prev_lineno = prev_tok.lineno
   else:
-    prev_lineno = prev_uwline.last.lineno
-  if cur_uwline.first.is_comment:
-    cur_lineno = cur_uwline.first.lineno - cur_uwline.first.value.count('\n')
+    prev_lineno = prev_tok.lineno
+
+  if cur_tok.is_comment:
+    cur_lineno = cur_tok.lineno - cur_tok.value.count('\n')
   else:
-    cur_lineno = cur_uwline.first.lineno
-  num_newlines = cur_lineno - prev_lineno
-  cur_uwline.first.AdjustNewlinesBefore(num_newlines)
+    cur_lineno = cur_tok.lineno
+
+  if prev_tok.value.endswith('\\'):
+    prev_lineno += prev_tok.value.count('\n')
+
+  required_newlines = cur_lineno - prev_lineno
+  if cur_tok.is_comment and not prev_tok.is_comment:
+    # Don't adjust between a comment and non-comment.
+    pass
+  elif lines and (cur_lineno in lines or prev_lineno in lines):
+    desired_newlines = cur_tok.whitespace_prefix.count('\n')
+    whitespace_lines = range(prev_lineno + 1, cur_lineno)
+    deletable_lines = len(lines.intersection(whitespace_lines))
+    required_newlines = max(required_newlines - deletable_lines,
+                            desired_newlines)
+
+  cur_tok.AdjustNewlinesBefore(required_newlines)
+
+
+def _RetainVerticalSpacingBeforeComments(uwline):
+  """Retain vertical spacing before comments."""
+  prev_token = None
+  for tok in uwline.tokens:
+    if tok.is_comment and prev_token:
+      if tok.lineno - tok.value.count('\n') - prev_token.lineno > 1:
+        tok.AdjustNewlinesBefore(ONE_BLANK_LINE)
+
+    prev_token = tok
 
 
 def _EmitLineUnformatted(state):
@@ -141,14 +178,14 @@ def _EmitLineUnformatted(state):
     previous_token = state.next_token.previous_token
     previous_lineno = previous_token.lineno
 
-    if previous_token.is_multiline_string:
+    if previous_token.is_multiline_string or previous_token.is_string:
       previous_lineno += previous_token.value.count('\n')
 
     if previous_token.is_continuation:
       newline = False
     else:
-      newline = (prev_lineno is not None and
-                 state.next_token.lineno > previous_lineno)
+      newline = (
+          prev_lineno is not None and state.next_token.lineno > previous_lineno)
 
     prev_lineno = state.next_token.lineno
     state.AddTokenToState(newline=newline, dry_run=False)
@@ -166,11 +203,11 @@ def _LineContainsI18n(uwline):
   Returns:
     True if the line contains i18n comments or function calls. False otherwise.
   """
-  if (style.Get('I18N_COMMENT') and any(re.search(style.Get('I18N_COMMENT'),
-                                                  tok.value)
-                                        for tok in uwline.tokens)):
-    # Contains an i18n comment.
-    return True
+  if style.Get('I18N_COMMENT'):
+    for tok in uwline.tokens:
+      if tok.is_comment and re.match(style.Get('I18N_COMMENT'), tok.value):
+        # Contains an i18n comment.
+        return True
 
   if style.Get('I18N_FUNCTION_CALL'):
     length = len(uwline.tokens)
@@ -199,8 +236,38 @@ def _CanPlaceOnSingleLine(uwline):
     True if the line can or should be added to a single line. False otherwise.
   """
   indent_amt = style.Get('INDENT_WIDTH') * uwline.depth
-  return (uwline.last.total_length + indent_amt <= style.Get('COLUMN_LIMIT') and
-          not any(tok.is_comment for tok in uwline.tokens[:-1]))
+  last = uwline.last
+  last_index = -1
+  if last.is_pylint_comment:
+    last = last.previous_token
+    last_index = -2
+  if last is None:
+    return True
+  return (last.total_length + indent_amt <= style.Get('COLUMN_LIMIT') and
+          not any(tok.is_comment for tok in uwline.tokens[:last_index]))
+
+
+def _FormatFinalLines(final_lines, verify):
+  """Compose the final output from the finalized lines."""
+  formatted_code = []
+  for line in final_lines:
+    formatted_line = []
+    for tok in line.tokens:
+      if not tok.is_pseudo_paren:
+        formatted_line.append(tok.whitespace_prefix)
+        formatted_line.append(tok.value)
+      else:
+        if (not tok.next_token.whitespace_prefix.startswith('\n') and
+            not tok.next_token.whitespace_prefix.startswith(' ')):
+          if (tok.previous_token.value == ':' or
+              tok.next_token.value not in ',}])'):
+            formatted_line.append(' ')
+
+    formatted_code.append(''.join(formatted_line))
+    if verify:
+      verifier.VerifyCode(formatted_code[-1])
+
+  return ''.join(formatted_code) + '\n'
 
 
 class _StateNode(object):
@@ -221,9 +288,10 @@ class _StateNode(object):
     self.newline = newline
     self.previous = previous
 
-  def __repr__(self):
-    return 'StateNode(state=[\n{0}\n], newline={1})'.format(self.state,
-                                                            self.newline)
+  def __repr__(self):  # pragma: no cover
+    return 'StateNode(state=[\n{0}\n], newline={1})'.format(
+        self.state, self.newline)
+
 
 # A tuple of (penalty, count) that is used to prioritize the BFS. In case of
 # equal penalties, we prefer states that were inserted first. During state
@@ -233,11 +301,11 @@ _OrderedPenalty = collections.namedtuple('OrderedPenalty', ['penalty', 'count'])
 
 # An item in the prioritized BFS search queue. The 'StateNode's 'state' has
 # the given '_OrderedPenalty'.
-_QueueItem = collections.namedtuple('QueueItem', ['ordered_penalty',
-                                                  'state_node'])
+_QueueItem = collections.namedtuple('QueueItem',
+                                    ['ordered_penalty', 'state_node'])
 
 
-def _AnalyzeSolutionSpace(initial_state, dry_run=False):
+def _AnalyzeSolutionSpace(initial_state):
   """Analyze the entire solution space starting from initial_state.
 
   This implements a variant of Dijkstra's algorithm on the graph that spans
@@ -248,7 +316,6 @@ def _AnalyzeSolutionSpace(initial_state, dry_run=False):
   Arguments:
     initial_state: (format_decision_state.FormatDecisionState) The initial state
       to start the search from.
-    dry_run: (bool) Don't commit changes if True.
 
   Returns:
     True if a formatting solution was found. False otherwise.
@@ -262,7 +329,6 @@ def _AnalyzeSolutionSpace(initial_state, dry_run=False):
   heapq.heappush(p_queue, _QueueItem(_OrderedPenalty(0, count), node))
 
   count += 1
-  prev_penalty = 0
   while p_queue:
     item = p_queue[0]
     penalty = item.ordered_penalty.penalty
@@ -277,9 +343,6 @@ def _AnalyzeSolutionSpace(initial_state, dry_run=False):
     if node.state in seen:
       continue
 
-    assert penalty >= prev_penalty
-    prev_penalty = penalty
-
     seen.add(node.state)
 
     # FIXME(morbo): Add a 'decision' element?
@@ -291,9 +354,7 @@ def _AnalyzeSolutionSpace(initial_state, dry_run=False):
     # We weren't able to find a solution. Do nothing.
     return False
 
-  if not dry_run:
-    _ReconstructPath(initial_state, heapq.heappop(p_queue).state_node)
-
+  _ReconstructPath(initial_state, heapq.heappop(p_queue).state_node)
   return True
 
 
@@ -314,15 +375,17 @@ def _AddNextStateToQueue(penalty, previous_node, newline, count, p_queue):
   Returns:
     The updated number of elements in the queue.
   """
-  if newline and not previous_node.state.CanSplit():
+  must_split = previous_node.state.MustSplit()
+  if newline and not previous_node.state.CanSplit(must_split):
     # Don't add a newline if the token cannot be split.
     return count
-  if not newline and previous_node.state.MustSplit():
+  if not newline and must_split:
     # Don't add a token we must split but where we aren't splitting.
     return count
 
   node = _StateNode(previous_node.state, newline, previous_node)
-  penalty += node.state.AddTokenToState(newline=newline, dry_run=True)
+  penalty += node.state.AddTokenToState(
+      newline=newline, dry_run=True, must_split=must_split)
   heapq.heappush(p_queue, _QueueItem(_OrderedPenalty(penalty, count), node))
   return count + 1
 
@@ -346,7 +409,7 @@ def _ReconstructPath(initial_state, current):
     initial_state.AddTokenToState(newline=node.newline, dry_run=False)
 
 
-def _FormatFirstToken(first_token, indent_depth, prev_uwline):
+def _FormatFirstToken(first_token, indent_depth, prev_uwline, final_lines):
   """Format the first token in the unwrapped line.
 
   Add a newline and the required indent before the first token of the unwrapped
@@ -358,11 +421,13 @@ def _FormatFirstToken(first_token, indent_depth, prev_uwline):
     indent_depth: (int) The line's indentation depth.
     prev_uwline: (list of unwrapped_line.UnwrappedLine) The unwrapped line
       previous to this line.
+    final_lines: (list of unwrapped_line.UnwrappedLine) The unwrapped lines
+      that have already been processed.
   """
-  first_token.AddWhitespacePrefix(_CalculateNumberOfNewlines(first_token,
-                                                             indent_depth,
-                                                             prev_uwline),
-                                  indent_level=indent_depth)
+  first_token.AddWhitespacePrefix(
+      _CalculateNumberOfNewlines(first_token, indent_depth, prev_uwline,
+                                 final_lines),
+      indent_level=indent_depth)
 
 
 NO_BLANK_LINES = 1
@@ -370,7 +435,15 @@ ONE_BLANK_LINE = 2
 TWO_BLANK_LINES = 3
 
 
-def _CalculateNumberOfNewlines(first_token, indent_depth, prev_uwline):
+def _IsClassOrDef(uwline):
+  if uwline.first.value in {'class', 'def'}:
+    return True
+
+  return [t.value for t in uwline.tokens[:2]] == ['async', 'def']
+
+
+def _CalculateNumberOfNewlines(first_token, indent_depth, prev_uwline,
+                               final_lines):
   """Calculate the number of newlines we need to add.
 
   Arguments:
@@ -379,6 +452,8 @@ def _CalculateNumberOfNewlines(first_token, indent_depth, prev_uwline):
     indent_depth: (int) The line's indentation depth.
     prev_uwline: (list of unwrapped_line.UnwrappedLine) The unwrapped line
       previous to this line.
+    final_lines: (list of unwrapped_line.UnwrappedLine) The unwrapped lines
+      that have already been processed.
 
   Returns:
     The number of newlines needed before the first token.
@@ -389,18 +464,21 @@ def _CalculateNumberOfNewlines(first_token, indent_depth, prev_uwline):
     # The first line in the file. Don't add blank lines.
     # FIXME(morbo): Is this correct?
     if first_token.newlines is not None:
-      pytree_utils.SetNodeAnnotation(first_token.GetPytreeNode(),
+      pytree_utils.SetNodeAnnotation(first_token.node,
                                      pytree_utils.Annotation.NEWLINES, None)
     return 0
 
   if first_token.is_docstring:
+    if (prev_uwline.first.value == 'class' and
+        style.Get('BLANK_LINE_BEFORE_CLASS_DOCSTRING')):
+      # Enforce a blank line before a class's docstring.
+      return ONE_BLANK_LINE
     # The docstring shouldn't have a newline before it.
-    # TODO(morbo): Add a knob to adjust this.
     return NO_BLANK_LINES
 
   prev_last_token = prev_uwline.last
   if prev_last_token.is_docstring:
-    if not indent_depth and first_token.value in {'class', 'def'}:
+    if (not indent_depth and first_token.value in {'class', 'def', 'async'}):
       # Separate a class or function from the module-level docstring with two
       # blank lines.
       return TWO_BLANK_LINES
@@ -410,25 +488,33 @@ def _CalculateNumberOfNewlines(first_token, indent_depth, prev_uwline):
     else:
       return ONE_BLANK_LINE
 
-  if first_token.value in {'class', 'def', '@'}:
+  if first_token.value in {'class', 'def', 'async', '@'}:
     # TODO(morbo): This can go once the blank line calculator is more
     # sophisticated.
     if not indent_depth:
       # This is a top-level class or function.
       is_inline_comment = prev_last_token.whitespace_prefix.count('\n') == 0
-      if prev_last_token.is_comment and not is_inline_comment:
+      if (not prev_uwline.disable and prev_last_token.is_comment and
+          not is_inline_comment):
         # This token follows a non-inline comment.
         if _NoBlankLinesBeforeCurrentToken(prev_last_token.value, first_token,
                                            prev_last_token):
           # Assume that the comment is "attached" to the current line.
           # Therefore, we want two blank lines before the comment.
-          prev_last_token.AdjustNewlinesBefore(TWO_BLANK_LINES)
+          index = len(final_lines) - 1
+          while index > 0:
+            if not final_lines[index - 1].is_comment:
+              break
+            index -= 1
+          if final_lines[index - 1].first.value == '@':
+            final_lines[index].first.AdjustNewlinesBefore(NO_BLANK_LINES)
+          else:
+            prev_last_token.AdjustNewlinesBefore(TWO_BLANK_LINES)
           if first_token.newlines is not None:
-            pytree_utils.SetNodeAnnotation(first_token.GetPytreeNode(),
-                                           pytree_utils.Annotation.NEWLINES,
-                                           None)
+            pytree_utils.SetNodeAnnotation(
+                first_token.node, pytree_utils.Annotation.NEWLINES, None)
           return NO_BLANK_LINES
-    elif prev_uwline.first.value in {'class', 'def'}:
+    elif _IsClassOrDef(prev_uwline):
       if not style.Get('BLANK_LINE_BEFORE_NESTED_CLASS_OR_DEF'):
         pytree_utils.SetNodeAnnotation(first_token.node,
                                        pytree_utils.Annotation.NEWLINES, None)
@@ -441,10 +527,14 @@ def _CalculateNumberOfNewlines(first_token, indent_depth, prev_uwline):
   else:
     first_token_lineno = first_token.lineno
 
-  if first_token_lineno - prev_last_token.lineno > 1:
+  prev_last_token_lineno = prev_last_token.lineno
+  if prev_last_token.is_multiline_string:
+    prev_last_token_lineno += prev_last_token.value.count('\n')
+
+  if first_token_lineno - prev_last_token_lineno > 1:
     return ONE_BLANK_LINE
-  else:
-    return NO_BLANK_LINES
+
+  return NO_BLANK_LINES
 
 
 def _SingleOrMergedLines(uwlines):
@@ -469,9 +559,7 @@ def _SingleOrMergedLines(uwlines):
           break
         if uwline.last.value != ':':
           leaf = pytree.Leaf(
-              type=token.SEMI,
-              value=';',
-              context=('', (uwline.lineno, column)))
+              type=token.SEMI, value=';', context=('', (uwline.lineno, column)))
           uwline.AppendToken(format_token.FormatToken(leaf))
         for tok in uwlines[index].tokens:
           uwline.AppendToken(tok)
