@@ -1,4 +1,4 @@
-# Copyright 2015 Google Inc. All Rights Reserved.
+# Copyright 2015-2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,8 +19,6 @@ single line if there were no line length restrictions. It's then used by the
 parser to perform the wrapping required to comply with the style guide.
 """
 
-from lib2to3 import pytree
-
 from yapf.yapflib import format_token
 from yapf.yapflib import py3compat
 from yapf.yapflib import pytree_utils
@@ -74,19 +72,44 @@ class UnwrappedLine(object):
           _SpaceRequiredBetween(prev_token, token)):
         token.spaces_required_before = 1
 
-      token.total_length = (
-          prev_length + len(token.value) + token.spaces_required_before)
+      tok_len = len(token.value) if not token.is_pseudo_paren else 0
+      token.total_length = prev_length + tok_len + token.spaces_required_before
 
       # The split penalty has to be computed before {must|can}_break_before,
       # because these may use it for their decision.
       token.split_penalty += _SplitPenalty(prev_token, token)
       token.must_break_before = _MustBreakBefore(prev_token, token)
-      token.can_break_before = (token.must_break_before or
-                                _CanBreakBefore(prev_token, token))
+      token.can_break_before = (
+          token.must_break_before or _CanBreakBefore(prev_token, token))
 
       prev_length = token.total_length
       prev_token = token
 
+  def Split(self):
+    """Split the line at semicolons."""
+    if not self.has_semicolon or self.disable:
+      return [self]
+
+    uwlines = []
+    uwline = UnwrappedLine(self.depth)
+    for tok in self._tokens:
+      if tok.value == ';':
+        uwlines.append(uwline)
+        uwline = UnwrappedLine(self.depth)
+      else:
+        uwline.AppendToken(tok)
+
+    if uwline.tokens:
+      uwlines.append(uwline)
+
+    for uwline in uwlines:
+      pytree_utils.SetNodeAnnotation(uwline.first.node,
+                                     pytree_utils.Annotation.MUST_SPLIT, True)
+      uwline.first.previous_token = None
+      uwline.last.next_token = None
+
+    return uwlines
+
   ############################################################################
   # Token Access and Manipulation Methods                                    #
   ############################################################################
@@ -106,7 +129,6 @@ class UnwrappedLine(object):
     Arguments:
       node: the node to append
     """
-    assert isinstance(node, pytree.Leaf)
     self.AppendToken(format_token.FormatToken(node))
 
   @property
@@ -142,14 +164,14 @@ class UnwrappedLine(object):
     tokens_str = ' '.join(tok.value for tok in self._tokens)
     return indent + tokens_str
 
-  def __str__(self):
+  def __str__(self):  # pragma: no cover
     return self.AsCode()
 
-  def __repr__(self):
-    tokens_repr = ','.join(['{0}({1!r})'.format(tok.name, tok.value)
-                            for tok in self._tokens])
-    return 'UnwrappedLine(depth={0}, tokens=[{1}])'.format(self.depth,
-                                                           tokens_repr)
+  def __repr__(self):  # pragma: no cover
+    tokens_repr = ','.join(
+        ['{0}({1!r})'.format(tok.name, tok.value) for tok in self._tokens])
+    return 'UnwrappedLine(depth={0}, tokens=[{1}])'.format(
+        self.depth, tokens_repr)
 
   ############################################################################
   # Properties                                                               #
@@ -179,6 +201,10 @@ class UnwrappedLine(object):
   def is_comment(self):
     return self.first.is_comment
 
+  @property
+  def has_semicolon(self):
+    return any(tok.value == ';' for tok in self._tokens)
+
 
 def _IsIdNumberStringToken(tok):
   return tok.is_keyword or tok.is_name or tok.is_number or tok.is_string
@@ -190,12 +216,17 @@ def _IsUnaryOperator(tok):
 
 def _SpaceRequiredBetween(left, right):
   """Return True if a space is required between the left and right token."""
+  lval = left.value
+  rval = right.value
   if (left.is_pseudo_paren and _IsIdNumberStringToken(right) and
       left.previous_token and _IsIdNumberStringToken(left.previous_token)):
     # Space between keyword... tokens and pseudo parens.
     return True
   if left.is_pseudo_paren or right.is_pseudo_paren:
-    # The pseudo-parens shouldn't affect spacing.
+    # There should be a space after the ':' in a dictionary.
+    if left.OpensScope():
+      return True
+    # The closing pseudo-paren shouldn't affect spacing.
     return False
   if left.is_continuation or right.is_continuation:
     # The continuation node's value has all of the spaces it needs.
@@ -206,110 +237,132 @@ def _SpaceRequiredBetween(left, right):
   if _IsIdNumberStringToken(left) and _IsIdNumberStringToken(right):
     # Spaces between keyword, string, number, and identifier tokens.
     return True
-  if left.value == ',' and right.value == ':':
+  if lval == ',' and rval == ':':
     # We do want a space between a comma and colon.
     return True
-  if right.value in ':,':
+  if rval in ':,':
     # Otherwise, we never want a space before a colon or comma.
     return False
-  if left.value == ',' and right.value in ']})':
+  if lval == ',' and rval in ']})':
     # Add a space between ending ',' and closing bracket if requested.
     return style.Get('SPACE_BETWEEN_ENDING_COMMA_AND_CLOSING_BRACKET')
-  if left.value == ',':
+  if lval == ',':
     # We want a space after a comma.
     return True
-  if left.value == 'from' and right.value == '.':
+  if lval == 'from' and rval == '.':
     # Space before the '.' in an import statement.
     return True
-  if left.value == '.' and right.value == 'import':
+  if lval == '.' and rval == 'import':
     # Space after the '.' in an import statement.
     return True
+  if lval == '=' and rval == '.':
+    # Space between equal and '.' as in "X = ...".
+    return True
   if ((right.is_keyword or right.is_name) and
       (left.is_keyword or left.is_name)):
     # Don't merge two keywords/identifiers.
     return True
-  if left.is_string and right.value not in '[)]}.':
-    # A string followed by something other than a subscript, closing bracket,
-    # or dot should have a space after it.
+  if (format_token.Subtype.SUBSCRIPT_COLON in left.subtypes or
+      format_token.Subtype.SUBSCRIPT_COLON in right.subtypes):
+    # A subscript shouldn't have spaces separating its colons.
+    return False
+  if (format_token.Subtype.TYPED_NAME in left.subtypes or
+      format_token.Subtype.TYPED_NAME in right.subtypes):
+    # A typed argument should have a space after the colon.
     return True
-  if left.is_binary_op and _IsUnaryOperator(right):
+  if left.is_string:
+    if (rval == '=' and
+        format_token.Subtype.DEFAULT_OR_NAMED_ASSIGN_ARG_LIST in right.subtypes
+       ):
+      # If there is a type hint, then we don't want to add a space between the
+      # equal sign and the hint.
+      return False
+    if rval not in '[)]}.':
+      # A string followed by something other than a subscript, closing bracket,
+      # or dot should have a space after it.
+      return True
+  if left.is_binary_op and lval != '**' and _IsUnaryOperator(right):
     # Space between the binary opertor and the unary operator.
     return True
+  if left.is_keyword and _IsUnaryOperator(right):
+    # Handle things like "not -3 < x".
+    return True
   if _IsUnaryOperator(left) and _IsUnaryOperator(right):
     # No space between two unary operators.
     return False
   if left.is_binary_op or right.is_binary_op:
-    if left.value == '**' or right.value == '**':
+    if lval == '**' or rval == '**':
       # Space around the "power" operator.
       return style.Get('SPACES_AROUND_POWER_OPERATOR')
-    # Enforce spaces around binary operators.
-    return True
-  if (_IsUnaryOperator(left) and left.value != 'not' and
-      (right.is_name or right.is_number or right.value == '(')):
+    # Enforce spaces around binary operators except the blacklisted ones.
+    blacklist = style.Get('NO_SPACES_AROUND_SELECTED_BINARY_OPERATORS')
+    return lval not in blacklist and rval not in blacklist
+  if (_IsUnaryOperator(left) and lval != 'not' and
+      (right.is_name or right.is_number or rval == '(')):
     # The previous token was a unary op. No space is desired between it and
     # the current token.
     return False
-  if (format_token.Subtype.SUBSCRIPT_COLON in left.subtypes or
-      format_token.Subtype.SUBSCRIPT_COLON in right.subtypes):
-    # A subscript shouldn't have spaces separating its colons.
-    return False
   if (format_token.Subtype.DEFAULT_OR_NAMED_ASSIGN in left.subtypes or
       format_token.Subtype.DEFAULT_OR_NAMED_ASSIGN in right.subtypes):
     # A named argument or default parameter shouldn't have spaces around it.
+    return style.Get('SPACES_AROUND_DEFAULT_OR_NAMED_ASSIGN')
+  if (format_token.Subtype.VARARGS_LIST in left.subtypes or
+      format_token.Subtype.VARARGS_LIST in right.subtypes):
     return False
   if (format_token.Subtype.VARARGS_STAR in left.subtypes or
       format_token.Subtype.KWARGS_STAR_STAR in left.subtypes):
     # Don't add a space after a vararg's star or a keyword's star-star.
     return False
-  if left.value == '@':
+  if lval == '@' and format_token.Subtype.DECORATOR in left.subtypes:
     # Decorators shouldn't be separated from the 'at' sign.
     return False
-  if left.value == '.' or right.value == '.':
+  if left.is_keyword and rval == '.' or lval == '.' and right.is_keyword:
+    # Add space between keywords and dots.
+    return lval != 'None'
+  if lval == '.' or rval == '.':
     # Don't place spaces between dots.
     return False
-  if ((left.value == '(' and right.value == ')') or
-      (left.value == '[' and right.value == ']') or
-      (left.value == '{' and right.value == '}')):
+  if ((lval == '(' and rval == ')') or (lval == '[' and rval == ']') or
+      (lval == '{' and rval == '}')):
     # Empty objects shouldn't be separted by spaces.
     return False
-  if (left.value in pytree_utils.OPENING_BRACKETS and
-      right.value in pytree_utils.OPENING_BRACKETS):
+  if (lval in pytree_utils.OPENING_BRACKETS and
+      rval in pytree_utils.OPENING_BRACKETS):
     # Nested objects' opening brackets shouldn't be separated.
     return False
-  if (left.value in pytree_utils.CLOSING_BRACKETS and
-      right.value in pytree_utils.CLOSING_BRACKETS):
+  if (lval in pytree_utils.CLOSING_BRACKETS and
+      rval in pytree_utils.CLOSING_BRACKETS):
     # Nested objects' closing brackets shouldn't be separated.
     return False
-  if left.value in pytree_utils.CLOSING_BRACKETS and right.value in '([':
+  if lval in pytree_utils.CLOSING_BRACKETS and rval in '([':
     # A call, set, dictionary, or subscript that has a call or subscript after
     # it shouldn't have a space between them.
     return False
-  if (left.value in pytree_utils.OPENING_BRACKETS and
-      _IsIdNumberStringToken(right)):
+  if lval in pytree_utils.OPENING_BRACKETS and _IsIdNumberStringToken(right):
     # Don't separate the opening bracket from the first item.
     return False
-  if left.is_name and right.value in '([':
+  if left.is_name and rval in '([':
     # Don't separate a call or array access from the name.
     return False
-  if right.value in pytree_utils.CLOSING_BRACKETS:
+  if rval in pytree_utils.CLOSING_BRACKETS:
     # Don't separate the closing bracket from the last item.
     # FIXME(morbo): This might be too permissive.
     return False
-  if left.value == 'print' and right.value == '(':
+  if lval == 'print' and rval == '(':
     # Special support for the 'print' function.
     return False
-  if left.value in pytree_utils.OPENING_BRACKETS and _IsUnaryOperator(right):
+  if lval in pytree_utils.OPENING_BRACKETS and _IsUnaryOperator(right):
     # Don't separate a unary operator from the opening bracket.
     return False
-  if (left.value in pytree_utils.OPENING_BRACKETS and
+  if (lval in pytree_utils.OPENING_BRACKETS and
       (format_token.Subtype.VARARGS_STAR in right.subtypes or
        format_token.Subtype.KWARGS_STAR_STAR in right.subtypes)):
     # Don't separate a '*' or '**' from the opening bracket.
     return False
-  if right.value == ';':
+  if rval == ';':
     # Avoid spaces before a semicolon. (Why is there a semicolon?!)
     return False
-  if left.value == '(' and right.value == 'await':
+  if lval == '(' and rval == 'await':
     # Special support for the 'await' keyword. Don't separate the 'await'
     # keyword from an opening paren.
     return False
@@ -318,48 +371,50 @@ def _SpaceRequiredBetween(left, right):
 
 def _MustBreakBefore(prev_token, cur_token):
   """Return True if a line break is required before the current token."""
-  if prev_token.is_comment:
+  if prev_token.is_comment or (prev_token.previous_token and
+                               prev_token.is_pseudo_paren and
+                               prev_token.previous_token.is_comment):
     # Must break if the previous token was a comment.
     return True
-  if (_IsSurroundedByBrackets(cur_token) and cur_token.is_string and
-      prev_token.is_string):
+  if (cur_token.is_string and prev_token.is_string and
+      IsSurroundedByBrackets(cur_token)):
     # We want consecutive strings to be on separate lines. This is a
     # reasonable assumption, because otherwise they should have written them
     # all on the same line, or with a '+'.
     return True
-  return pytree_utils.GetNodeAnnotation(cur_token.node,
-                                        pytree_utils.Annotation.MUST_SPLIT,
-                                        default=False)
+  return pytree_utils.GetNodeAnnotation(
+      cur_token.node, pytree_utils.Annotation.MUST_SPLIT, default=False)
 
 
 def _CanBreakBefore(prev_token, cur_token):
   """Return True if a line break may occur before the current token."""
+  pval = prev_token.value
+  cval = cur_token.value
   if py3compat.PY3:
-    if prev_token.value == 'yield' and cur_token.value == 'from':
+    if pval == 'yield' and cval == 'from':
       # Don't break before a yield argument.
       return False
-    if (prev_token.value in {'async', 'await'} and
-        cur_token.value in {'def', 'with', 'for'}):
+    if pval in {'async', 'await'} and cval in {'def', 'with', 'for'}:
       # Don't break after sync keywords.
       return False
   if cur_token.split_penalty >= split_penalty.UNBREAKABLE:
     return False
-  if prev_token.value == '@':
+  if pval == '@':
     # Don't break right after the beginning of a decorator.
     return False
-  if cur_token.value == ':':
+  if cval == ':':
     # Don't break before the start of a block of code.
     return False
-  if cur_token.value == ',':
+  if cval == ',':
     # Don't break before a comma.
     return False
-  if prev_token.is_name and cur_token.value == '(':
+  if prev_token.is_name and cval == '(':
     # Don't break in the middle of a function definition or call.
     return False
-  if prev_token.is_name and cur_token.value == '[':
+  if prev_token.is_name and cval == '[':
     # Don't break in the middle of an array dereference.
     return False
-  if prev_token.is_name and cur_token.value == '.':
+  if prev_token.is_name and cval == '.':
     # Don't break before the '.' in a dotted name.
     return False
   if cur_token.is_comment and prev_token.lineno == cur_token.lineno:
@@ -371,7 +426,7 @@ def _CanBreakBefore(prev_token, cur_token):
   return True
 
 
-def _IsSurroundedByBrackets(tok):
+def IsSurroundedByBrackets(tok):
   """Return True if the token is surrounded by brackets."""
   paren_count = 0
   brace_count = 0
@@ -387,19 +442,19 @@ def _IsSurroundedByBrackets(tok):
 
     if previous_token.value == '(':
       if paren_count == 0:
-        return True
+        return previous_token
       paren_count += 1
     elif previous_token.value == '{':
       if brace_count == 0:
-        return True
+        return previous_token
       brace_count += 1
     elif previous_token.value == '[':
       if sq_bracket_count == 0:
-        return True
+        return previous_token
       sq_bracket_count += 1
 
     previous_token = previous_token.previous_token
-  return False
+  return None
 
 
 _LOGICAL_OPERATORS = frozenset({'and', 'or'})
@@ -409,7 +464,9 @@ _TERM_OPERATORS = frozenset({'*', '/', '%', '//'})
 
 def _SplitPenalty(prev_token, cur_token):
   """Return the penalty for breaking the line before the current token."""
-  if prev_token.value == 'not':
+  pval = prev_token.value
+  cval = cur_token.value
+  if pval == 'not':
     return split_penalty.UNBREAKABLE
 
   if cur_token.node_split_penalty > 0:
@@ -417,28 +474,28 @@ def _SplitPenalty(prev_token, cur_token):
 
   if style.Get('SPLIT_BEFORE_LOGICAL_OPERATOR'):
     # Prefer to split before 'and' and 'or'.
-    if prev_token.value in _LOGICAL_OPERATORS:
+    if pval in _LOGICAL_OPERATORS:
       return style.Get('SPLIT_PENALTY_LOGICAL_OPERATOR')
-    if cur_token.value in _LOGICAL_OPERATORS:
+    if cval in _LOGICAL_OPERATORS:
       return 0
   else:
     # Prefer to split after 'and' and 'or'.
-    if prev_token.value in _LOGICAL_OPERATORS:
+    if pval in _LOGICAL_OPERATORS:
       return 0
-    if cur_token.value in _LOGICAL_OPERATORS:
+    if cval in _LOGICAL_OPERATORS:
       return style.Get('SPLIT_PENALTY_LOGICAL_OPERATOR')
 
   if style.Get('SPLIT_BEFORE_BITWISE_OPERATOR'):
     # Prefer to split before '&', '|', and '^'.
-    if prev_token.value in _BITWISE_OPERATORS:
+    if pval in _BITWISE_OPERATORS:
       return style.Get('SPLIT_PENALTY_BITWISE_OPERATOR')
-    if cur_token.value in _BITWISE_OPERATORS:
+    if cval in _BITWISE_OPERATORS:
       return 0
   else:
     # Prefer to split after '&', '|', and '^'.
-    if prev_token.value in _BITWISE_OPERATORS:
+    if pval in _BITWISE_OPERATORS:
       return 0
-    if cur_token.value in _BITWISE_OPERATORS:
+    if cval in _BITWISE_OPERATORS:
       return style.Get('SPLIT_PENALTY_BITWISE_OPERATOR')
 
   if (format_token.Subtype.COMP_FOR in cur_token.subtypes or
@@ -448,7 +505,7 @@ def _SplitPenalty(prev_token, cur_token):
   if format_token.Subtype.UNARY_OPERATOR in prev_token.subtypes:
     # Try not to break after a unary operator.
     return style.Get('SPLIT_PENALTY_AFTER_UNARY_OPERATOR')
-  if prev_token.value == ',':
+  if pval == ',':
     # Breaking after a comma is fine, if need be.
     return 0
   if prev_token.is_binary_op:
@@ -458,25 +515,25 @@ def _SplitPenalty(prev_token, cur_token):
       format_token.Subtype.KWARGS_STAR_STAR in prev_token.subtypes):
     # Don't split after a varargs * or kwargs **.
     return split_penalty.UNBREAKABLE
-  if prev_token.value in pytree_utils.OPENING_BRACKETS:
+  if prev_token.OpensScope() and cval != '(':
     # Slightly prefer
     return style.Get('SPLIT_PENALTY_AFTER_OPENING_BRACKET')
-  if cur_token.value == ':':
+  if cval == ':':
     # Don't split before a colon.
     return split_penalty.UNBREAKABLE
-  if cur_token.value == '=':
+  if cval == '=':
     # Don't split before an assignment.
     return split_penalty.UNBREAKABLE
   if (format_token.Subtype.DEFAULT_OR_NAMED_ASSIGN in prev_token.subtypes or
       format_token.Subtype.DEFAULT_OR_NAMED_ASSIGN in cur_token.subtypes):
     # Don't break before or after an default or named assignment.
     return split_penalty.UNBREAKABLE
-  if cur_token.value == '==':
+  if cval == '==':
     # We would rather not split before an equality operator.
     return split_penalty.STRONGLY_CONNECTED
   if cur_token.ClosesScope():
     # Give a slight penalty for splitting before the closing scope.
     return 100
-  if prev_token.value in _TERM_OPERATORS or cur_token.value in _TERM_OPERATORS:
+  if pval in _TERM_OPERATORS or cval in _TERM_OPERATORS:
     return 50
   return 0
