@@ -1,4 +1,4 @@
-# Copyright 2015 Google Inc. All Rights Reserved.
+# Copyright 2015-2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,8 +24,9 @@ from yapf.yapflib import pytree_visitor
 from yapf.yapflib import split_penalty
 
 UNBREAKABLE = split_penalty.UNBREAKABLE
+VERY_STRONGLY_CONNECTED = split_penalty.VERY_STRONGLY_CONNECTED
+DOTTED_NAME = split_penalty.DOTTED_NAME
 STRONGLY_CONNECTED = split_penalty.STRONGLY_CONNECTED
-CONTIGUOUS_LIST = split_penalty.CONTIGUOUS_LIST
 
 
 class SplitPenaltyTest(unittest.TestCase):
@@ -60,8 +61,9 @@ class SplitPenaltyTest(unittest.TestCase):
       if pytree_utils.NodeName(tree) in pytree_utils.NONSEMANTIC_TOKENS:
         return []
       if isinstance(tree, pytree.Leaf):
-        return [(tree.value, pytree_utils.GetNodeAnnotation(
-            tree, pytree_utils.Annotation.SPLIT_PENALTY))]
+        return [(tree.value,
+                 pytree_utils.GetNodeAnnotation(
+                     tree, pytree_utils.Annotation.SPLIT_PENALTY))]
       nodes = []
       for node in tree.children:
         nodes += FlattenRec(node)
@@ -81,10 +83,10 @@ class SplitPenaltyTest(unittest.TestCase):
         ('foo', UNBREAKABLE),
         ('(', UNBREAKABLE),
         ('x', None),
-        (')', None),
+        (')', STRONGLY_CONNECTED),
         (':', UNBREAKABLE),
         ('pass', None),
-    ])  # yapf: disable
+    ])
 
     # Test function definition with trailing comment.
     code = textwrap.dedent(r"""
@@ -97,10 +99,10 @@ class SplitPenaltyTest(unittest.TestCase):
         ('foo', UNBREAKABLE),
         ('(', UNBREAKABLE),
         ('x', None),
-        (')', None),
+        (')', STRONGLY_CONNECTED),
         (':', UNBREAKABLE),
         ('pass', None),
-    ])  # yapf: disable
+    ])
 
     # Test class definitions.
     code = textwrap.dedent(r"""
@@ -122,7 +124,7 @@ class SplitPenaltyTest(unittest.TestCase):
         (')', None),
         (':', UNBREAKABLE),
         ('pass', None),
-    ])  # yapf: disable
+    ])
 
     # Test lambda definitions.
     code = textwrap.dedent(r"""
@@ -136,7 +138,7 @@ class SplitPenaltyTest(unittest.TestCase):
         ('b', UNBREAKABLE),
         (':', UNBREAKABLE),
         ('None', UNBREAKABLE),
-    ])  # yapf: disable
+    ])
 
     # Test dotted names.
     code = textwrap.dedent(r"""
@@ -150,7 +152,7 @@ class SplitPenaltyTest(unittest.TestCase):
         ('b', UNBREAKABLE),
         ('.', UNBREAKABLE),
         ('c', UNBREAKABLE),
-    ])  # yapf: disable
+    ])
 
   def testStronglyConnected(self):
     # Test dictionary keys.
@@ -162,23 +164,25 @@ class SplitPenaltyTest(unittest.TestCase):
       """)
     tree = self._ParseAndComputePenalties(code)
     self._CheckPenalties(tree, [
-        ('a', None), ('=', None), ('{', None),
-        ("'x'", STRONGLY_CONNECTED),
+        ('a', None),
+        ('=', None),
+        ('{', None),
+        ("'x'", None),
         (':', STRONGLY_CONNECTED),
         ('42', None),
         (',', None),
-        ('y', STRONGLY_CONNECTED),
+        ('y', None),
         ('(', UNBREAKABLE),
         ('lambda', STRONGLY_CONNECTED),
         ('a', UNBREAKABLE),
         (':', UNBREAKABLE),
         ('23', UNBREAKABLE),
-        (')', UNBREAKABLE),
+        (')', VERY_STRONGLY_CONNECTED),
         (':', STRONGLY_CONNECTED),
         ('37', None),
         (',', None),
         ('}', None),
-    ])  # yapf: disable
+    ])
 
     # Test list comprehension.
     code = textwrap.dedent(r"""
@@ -188,18 +192,18 @@ class SplitPenaltyTest(unittest.TestCase):
     self._CheckPenalties(tree, [
         ('[', None),
         ('a', None),
-        ('for', None),
+        ('for', 0),
         ('a', STRONGLY_CONNECTED),
         ('in', STRONGLY_CONNECTED),
         ('foo', STRONGLY_CONNECTED),
         ('if', 0),
         ('a', STRONGLY_CONNECTED),
         ('.', UNBREAKABLE),
-        ('x', UNBREAKABLE),
+        ('x', DOTTED_NAME),
         ('==', STRONGLY_CONNECTED),
         ('37', STRONGLY_CONNECTED),
         (']', None),
-    ])  # yapf: disable
+    ])
 
   def testFuncCalls(self):
     code = 'foo(1, 2, 3)\n'
@@ -207,12 +211,13 @@ class SplitPenaltyTest(unittest.TestCase):
     self._CheckPenalties(tree, [
         ('foo', None),
         ('(', UNBREAKABLE),
-        ('1', CONTIGUOUS_LIST),
-        (',', CONTIGUOUS_LIST),
-        ('2', CONTIGUOUS_LIST),
-        (',', CONTIGUOUS_LIST),
-        ('3', CONTIGUOUS_LIST),
-        (')', UNBREAKABLE)])  # yapf: disable
+        ('1', None),
+        (',', UNBREAKABLE),
+        ('2', None),
+        (',', UNBREAKABLE),
+        ('3', None),
+        (')', VERY_STRONGLY_CONNECTED),
+    ])
 
     # Now a method call, which has more than one trailer
     code = 'foo.bar.baz(1, 2, 3)\n'
@@ -220,16 +225,34 @@ class SplitPenaltyTest(unittest.TestCase):
     self._CheckPenalties(tree, [
         ('foo', None),
         ('.', UNBREAKABLE),
-        ('bar', UNBREAKABLE),
-        ('.', UNBREAKABLE),
-        ('baz', UNBREAKABLE),
+        ('bar', DOTTED_NAME),
+        ('.', STRONGLY_CONNECTED),
+        ('baz', DOTTED_NAME),
+        ('(', STRONGLY_CONNECTED),
+        ('1', None),
+        (',', UNBREAKABLE),
+        ('2', None),
+        (',', UNBREAKABLE),
+        ('3', None),
+        (')', VERY_STRONGLY_CONNECTED),
+    ])
+
+    # Test single generator argument.
+    code = 'max(i for i in xrange(10))\n'
+    tree = self._ParseAndComputePenalties(code)
+    self._CheckPenalties(tree, [
+        ('max', None),
         ('(', UNBREAKABLE),
-        ('1', CONTIGUOUS_LIST),
-        (',', CONTIGUOUS_LIST),
-        ('2', CONTIGUOUS_LIST),
-        (',', CONTIGUOUS_LIST),
-        ('3', CONTIGUOUS_LIST),
-        (')', UNBREAKABLE)])  # yapf: disable
+        ('i', 0),
+        ('for', 0),
+        ('i', STRONGLY_CONNECTED),
+        ('in', STRONGLY_CONNECTED),
+        ('xrange', STRONGLY_CONNECTED),
+        ('(', UNBREAKABLE),
+        ('10', STRONGLY_CONNECTED),
+        (')', VERY_STRONGLY_CONNECTED),
+        (')', VERY_STRONGLY_CONNECTED),
+    ])
 
 
 if __name__ == '__main__':
